@@ -192,12 +192,13 @@ PhoneticEditor::processCommit (guint keyval, guint keycode,
         return FALSE;
 
     if (cmshm_filter (modifiers) != 0)
-        return TRUE;
+        return FALSE;
 
     if (!m_text)
         return FALSE;
 
     commit ();
+    update ();
     return TRUE;
 }
 
@@ -701,20 +702,12 @@ PhoneticEditor::getCursorLeft (void)
 
         /* only when in parsed phonetic section, need adjustments.  */
         if (cursor < parsed_len) {
-            guint16 offset = 0;
-            zhuyin_get_zhuyin_key_rest_offset (instance, cursor, &offset);
-
-            /* move to the begin of current syllable. */
-            ChewingKeyRest * key_rest = NULL;
-            zhuyin_get_zhuyin_key_rest (instance, offset, &key_rest);
-
-            guint16 begin = 0;
-            zhuyin_get_zhuyin_key_rest_positions
-                (instance, key_rest, &begin, NULL);
+            size_t offset = 0;
+            zhuyin_get_zhuyin_offset (instance, cursor, &offset);
 
             /* align to the begin of chewing key. */
             /* restore cursor variable. */
-            return m_cursor - (cursor + 1 - begin);
+            return m_cursor - (cursor + 1) + offset;
         }
     }
 
@@ -750,29 +743,13 @@ PhoneticEditor::getCursorRight (void)
 
         /* when in parsed phonetic section, need adjustments.  */
         if (cursor < parsed_len) {
-            guint16 offset = 0;
-            zhuyin_get_zhuyin_key_rest_offset (instance, cursor, &offset);
-            offset ++;
+            size_t offset = 0;
+            zhuyin_get_zhuyin_offset (instance, cursor, &offset);
 
-            guint len = 0;
-            zhuyin_get_n_zhuyin (instance, &len);
+            size_t right = 0;
+            zhuyin_get_right_zhuyin_offset(instance, offset, &right);
 
-            if (offset < len) {
-                /* move to the begin of next syllable. */
-                ChewingKeyRest * key_rest = NULL;
-                zhuyin_get_zhuyin_key_rest (instance, offset, &key_rest);
-
-                guint16 begin = 0;
-                zhuyin_get_zhuyin_key_rest_positions
-                (instance, key_rest, &begin, NULL);
-
-                /* align to the begin of chewing key. */
-                return m_cursor + (begin - cursor);
-            } else {
-                assert (offset == len);
-                /* align to the end of parsed phonetic section. */
-                return m_cursor + (parsed_len - cursor);
-            }
+            return m_cursor - cursor + right;
         }
     }
 
@@ -912,17 +889,18 @@ PhoneticEditor::getZhuyinCursor (void)
 
             assert (parsed_len <= section_len);
 
+            gchar * sentence = NULL;
+            zhuyin_get_sentence (instance, &sentence);
             if (cursor >= parsed_len) {
                 cursor -= parsed_len;
-                guint len = 0;
-                zhuyin_get_n_zhuyin (instance, &len);
-                zhuyin_cursor += len;
+                if (sentence)
+                    zhuyin_cursor += g_utf8_strlen (sentence, -1);
             } else {
-                guint16 inner_cursor = 0;
-                zhuyin_get_zhuyin_key_rest_offset
-                    (instance, cursor, &inner_cursor);
+                size_t length = 0;
+                zhuyin_get_character_offset
+                    (instance, sentence, cursor, &length);
+                zhuyin_cursor += length;
 
-                zhuyin_cursor += inner_cursor;
                 cursor = 0;
             }
 
@@ -938,6 +916,8 @@ PhoneticEditor::getZhuyinCursor (void)
                 cursor = 0;
             }
 
+            g_free (sentence);
+
             ++ index;
         }
 
@@ -1001,13 +981,21 @@ PhoneticEditor::insertSymbol (guint keyval)
 gboolean
 PhoneticEditor::insertNumbers (guint keyval)
 {
-    /* for input pad numbers. */
-    static const guint keyvals[] = {IBUS_KP_Delete, IBUS_KP_Insert,
-                                    IBUS_KP_End, IBUS_KP_Down,
-                                    IBUS_KP_Next, IBUS_KP_Left,
-                                    IBUS_KP_Begin, IBUS_KP_Right,
-                                    IBUS_KP_Home, IBUS_KP_Up,
-                                    IBUS_KP_Prior};
+    /* for input pad numbers with num lock on. */
+    static const guint num_lock_on[] = {IBUS_KP_Decimal, IBUS_KP_0,
+                                        IBUS_KP_1, IBUS_KP_2,
+                                        IBUS_KP_3, IBUS_KP_4,
+                                        IBUS_KP_5, IBUS_KP_6,
+                                        IBUS_KP_7, IBUS_KP_8,
+                                        IBUS_KP_9};
+
+    /* for input pad numbers with num lock off. */
+    static const guint num_lock_off[] = {IBUS_KP_Delete, IBUS_KP_Insert,
+                                         IBUS_KP_End, IBUS_KP_Down,
+                                         IBUS_KP_Next, IBUS_KP_Left,
+                                         IBUS_KP_Begin, IBUS_KP_Right,
+                                         IBUS_KP_Home, IBUS_KP_Up,
+                                         IBUS_KP_Prior};
 
     static const char numbers[] = {'.', '0',
                                    '1', '2',
@@ -1016,13 +1004,12 @@ PhoneticEditor::insertNumbers (guint keyval)
                                    '7', '8',
                                    '9'};
 
-    assert (G_N_ELEMENTS (keyvals) == G_N_ELEMENTS (numbers));
+    assert (G_N_ELEMENTS (num_lock_on) == G_N_ELEMENTS (numbers));
+    assert (G_N_ELEMENTS (num_lock_off) == G_N_ELEMENTS (numbers));
 
-    if (!m_config.alwaysInputNumbers ())
-        return FALSE;
-
-    for (size_t i = 0; i < G_N_ELEMENTS (keyvals); ++i) {
-        if (keyvals[i] == keyval) {
+    for (size_t i = 0; i < G_N_ELEMENTS (numbers); ++i) {
+        if (num_lock_on[i] == keyval ||
+            (m_config.alwaysInputNumbers () && num_lock_off[i] == keyval)) {
             String choice;
             choice += numbers[i];
 
