@@ -74,9 +74,8 @@ public:
     virtual void updateLookupTableLabel ();
     virtual gboolean fillLookupTableByPage ();
 
-    virtual gboolean insertPunct (guint ch);
-    virtual gboolean insertEnglish (guint ch);
-    virtual gboolean insertNumbers (guint ch);
+    virtual gboolean insertSymbol (guint keyval);
+    virtual gboolean insertNumbers (guint keyval);
 
 protected:
     gboolean prepareCandidates (void);
